/*
 *  $Id: dmpiatom.h,v 1.8 1993/11/12 11:37:32 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/* Atomic types */

#ifndef _DMPIATOM_INCLUDE
#define _DMPIATOM_INCLUDE

/*****************************************************************************
*                           MPI ATOMIC DATA STRUCTURES                             *
*****************************************************************************/

/* insure that this is at least 32 bits */
typedef unsigned long uint32;

typedef struct _MPIR_HBT_node {
  void *value;
  int   keyval;
  short balance;
  struct _MPIR_HBT_node *left;
  struct _MPIR_HBT_node *right;
} MPIR_HBT_node;

typedef struct _MPIR_HBT {
  unsigned int height;
  MPIR_HBT_node *root;
} MPIR_HBT;

typedef struct  {
  int  (*copy_fn)();
  int  (*delete_fn)();
  void  *extra_state;
  int    ref_count;
} MPIR_Attr_key;

typedef struct {
  int nnodes;
  int nedges;
  int *index;
  int *edges;
} MPIR_GRAPH_TOPOLOGY;

typedef struct {
  int nnodes;
  int ndims;
  int *dims;
  int *periods;
  int *position;
} MPIR_CART_TOPOLOGY;

typedef struct {
  int type;
  union {
    MPIR_GRAPH_TOPOLOGY  graph;
    MPIR_CART_TOPOLOGY   cart;
  } tag;
} MPIR_TOPOLOGY;

typedef struct {
    int np;			        /* Number of processes in group */
    int local_rank;         /* My rank in the group (if I belong) */
    int ref_count;          /* Number of references to this group */
    int N2_next;            /* Next power of 2 from np */
    int N2_prev;            /* Previous power of 2 from np */
    uint32 *lrank_to_grank;	/* Mapping from local to "global" ranks */
    uint32 *set_mark;       /* Used for set marking/manipulation on groups */
} MPIR_GROUP;

typedef unsigned long MPIR_CONTEXT;
#define  MPIR_PT2PT_CONTEXT 0
#define  MPIR_COLL_CONTEXT  1
#define  MPIR_FIRST_FREE_CONTEXT 2

typedef enum { 
    MPI_ERRORS_FATAL = 0, MPI_ERRORS_RETURN, MPI_ERRORS_CALL_USER } 
MPI_Error_mode;

typedef enum { MPIR_INTRA=0, MPIR_INTER } MPIR_COMM_TYPE;


typedef struct _MPIR_COMMUNICATOR {
    MPIR_COMM_TYPE comm_type;	/* inter or intra */
    MPIR_GROUP    *group;	/* group associated with communicator */
    struct _MPIR_COMMUNICATOR *comm_coll; /* communicator for collective ops */

    int            ref_count;  /* number of references to communicator */
    MPIR_CONTEXT   send_context;  /* context to send messages */
    MPIR_CONTEXT   recv_context;  /* context to recv messages */
    void          *comm_cache;	/* Hook for communicator cache */
    MPIR_HBT      *attr_cache;   /* Hook for attribute cache */
    MPIR_TOPOLOGY  topology;     /* Topology information */
    MPI_Error_mode error_mode;      /* Error mode */
    int          (*error_handler)();  /* User routine to call */
} MPIR_COMMUNICATOR;

typedef enum {
    NO = 0, YES
} MPIR_BOOL;

typedef enum {
    MPIR_INT, MPIR_FLOAT, MPIR_DOUBLE, MPIR_LONG, MPIR_SHORT, MPIR_CHAR,
    MPIR_BYTE, MPIR_UCHAR, MPIR_USHORT, MPIR_ULONG, MPIR_UINT,
    MPIR_CONTIG, MPIR_VECTOR, MPIR_HVECTOR, MPIR_INDEXED,
    MPIR_HINDEXED, MPIR_STRUCT
} MPIR_NODETYPE;

#include "mpi_bc.h"
#endif
