/*
 *  $Id: alltoall.c,v 1.4 1993/11/12 12:30:51 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: alltoall.c,v 1.4 1993/11/12 12:30:51 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Alltoall - 

@*/
int MPI_Alltoall(sendbuf,sendcnt,sendtype,recvbuf,recvcnt,recvtype,comm)
void             *sendbuf;
int               sendcnt;
MPI_Datatype      sendtype;
void             *recvbuf;
int               recvcnt;
MPI_Datatype      recvtype;
MPI_Comm  comm;
{
  int              size, rank, i, extent;
  MPI_Comm_request req;
  MPI_Status       status;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (comm->comm_type == MPIR_INTER) ) 
    return (MPI_SUCCESS); 
  
  /* Get size and rank and switch to collective communicator */
  MPI_Comm_size ( comm, &size );
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* Send to all - receive from all */
  MPI_Extent(recvtype, &extent);
  for ( i=0; i<size; i++ ) { 
    if (i >= rank) {
      MPI_Isend( (void *)((int)sendbuf+i*sendcnt*extent), sendcnt, sendtype, i,
		 MPIR_ALLTOALL_TAG, comm, &req );
      MPI_Wait ( req, &status );
      MPI_Irecv( (void *)((int)recvbuf+i*recvcnt*extent), recvcnt, recvtype, i, 
		 MPIR_ALLTOALL_TAG, comm, &req);
      MPI_Wait ( req, &status );
    }
    else {
      MPI_Irecv( (void *)((int)recvbuf+i*recvcnt*extent), recvcnt, recvtype, i, 
		 MPIR_ALLTOALL_TAG, comm, &req);
      MPI_Wait ( req, &status );
      MPI_Isend( (void *)((int)sendbuf+i*sendcnt*extent), sendcnt, sendtype, i,
		 MPIR_ALLTOALL_TAG, comm, &req );
      MPI_Wait ( req, &status );
    }
  }
  return (MPI_SUCCESS);
}
