/*
 *  $Id: alltoallv.c,v 1.4 1993/11/12 12:30:51 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: alltoallv.c,v 1.4 1993/11/12 12:30:51 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Alltoallv - 

@*/
int MPI_Alltoallv ( sendbuf, sdispls, sendcnts, sendtype, 
                    recvbuf, rdispls, recvcnts, recvtype, comm )
void             *sendbuf;
int              *sdispls;
int              *sendcnts;
MPI_Datatype      sendtype;
void             *recvbuf;
int              *rdispls; 
int              *recvcnts;
MPI_Datatype      recvtype;
MPI_Comm  comm;
{
  int              size, rank, i, extent;
  MPI_Comm_request req;
  MPI_Status       status;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (comm->comm_type == MPIR_INTER) ) 
    return (MPI_SUCCESS); 
  
  /* Get size and rank and switch to collective communicator */
  MPI_Comm_size ( comm, &size );
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* Send to all - receive from all */
  MPI_Extent(recvtype, &extent);
  for ( i=0; i<size; i++ ) { 
    if (i >= rank) {
      MPI_Isend( (void *)((int)sendbuf+sdispls[i]*extent), sendcnts[i], sendtype, i,
		 MPIR_ALLTOALLV_TAG, comm, &req );
      MPI_Wait ( req, &status );
      MPI_Irecv( (void *)((int)recvbuf+rdispls[i]*extent), recvcnts[i], recvtype, i, 
		 MPIR_ALLTOALLV_TAG, comm, &req);
      MPI_Wait ( req, &status );
    }
    else {
      MPI_Irecv( (void *)((int)recvbuf+rdispls[i]*extent), recvcnts[i], recvtype, i, 
		 MPIR_ALLTOALLV_TAG, comm, &req);
      MPI_Wait ( req, &status );
      MPI_Isend( (void *)((int)sendbuf+sdispls[i]*extent), sendcnts[i], sendtype, i,
		 MPIR_ALLTOALLV_TAG, comm, &req );
      MPI_Wait ( req, &status );
    }
  }

  return (MPI_SUCCESS);
}
