/*
 *  $Id: gather.c,v 1.4 1993/11/12 12:30:51 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: gather.c,v 1.4 1993/11/12 12:30:51 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Gather - 

@*/
int MPI_Gather ( sendbuf, sendcnt, sendtype, recvbuf, recvcnt, recvtype, 
		 root, comm )
void             *sendbuf;
int               sendcnt;
MPI_Datatype      sendtype;
void             *recvbuf;
int               recvcnt;
MPI_Datatype      recvtype;
int               root;
MPI_Comm  comm;
{
  int size, rank;
  MPI_Comm_request req;
  MPI_Status       status;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0) ||
       (root            <  0)                   || 
       (comm->comm_type == MPIR_INTER) ) 
    return (MPI_SUCCESS); 
  
  /* Is root within the communicator? */
  MPI_Comm_size ( comm, &size );
  if ( root >= size )
    return (MPI_SUCCESS);

  /* Get my rank and switch communicators to the hidden collective */
  MPI_Comm_rank ( comm, &rank );
  comm = comm->comm_coll;

  /* If rank == root, then I recv lots, otherwise I send */
  if ( rank == root ) {
    int i, extent;

    MPI_Isend(sendbuf, sendcnt, sendtype, root, MPIR_GATHER_TAG, comm, &req);
    MPI_Wait (req, &status );

    MPI_Extent(recvtype, &extent);
    for ( i=0; i<size; i++ ) {
      MPI_Irecv( (void *)((int)recvbuf+i*extent*recvcnt), recvcnt, recvtype, i, 
		 MPIR_GATHER_TAG, comm, &req);
      MPI_Wait ( req, &status );
    }
  }
  else {
    MPI_Isend(sendbuf, sendcnt, sendtype, root, MPIR_GATHER_TAG, comm, &req);
    MPI_Wait (req, &status );
  }
  
  return (MPI_SUCCESS);
}

