/*
 *  $Id: cart_coords.c,v 1.4 1993/11/12 12:35:46 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_coords - determines process coords in cartesian topology given
                  rank in group

@*/
int MPI_Cart_coords ( comm, rank, maxdims, coords )
MPI_Comm  comm;
int               rank;
int               maxdims;
int              *coords;
{
  int i, ndims, *dims, num_ranks;

  /* Check for valid arguments */
  if ( (comm                      == (MPI_Comm)0)              ||
       (comm->topology.type       != MPI_CART)                         ||
       (rank                      <  0)                                ||
       (rank                      >= (comm->topology.tag.cart.nnodes)) ||
       (maxdims                   <  1)                                ||
       (coords                    == (int *)0) )
    return (MPI_SUCCESS);

  /* Calculate coords */
  ndims     = comm->topology.tag.cart.ndims;
  dims      = comm->topology.tag.cart.dims;
  num_ranks = comm->topology.tag.cart.nnodes;
  for ( i=0; i < ndims; i++ ) {
    num_ranks = num_ranks / dims[i];
    if ( i < maxdims )
      coords[i]  = rank / num_ranks;
    rank   = rank % num_ranks;
  }

  return (MPI_SUCCESS);
}
